#include "main.h"

/*
Game harness 0.1

Work in progress...

Written by Bamse and G0blin 2004...
bamse@playstation2-linux.com
g0blin@playstation2-linux.com
*/

// -=*=-=*=-=*=-=*=-=*=-=*=-=*=- Global variables -=*=-=*=-=*=-=*=-=*=-

// The SPS2 handle.
int iSPS2Desc;
// Keeps track of wheter we are in Menu, Game Play or Game Over mode.
int gameMode = 0;

// Highscore and scores, always count score with the Score variable and then set Score1=Score or Score2=Score depending on the players turn.
int hiScore = 1968;
int Score = 0;
int Score1 = 0;
int Score2 = 0;

// Player one or two.
int player = 1;

// General counter.
int counter = 0;

// Memory Managers
CSPS2Manager Manager;
char * pPrimitiveMem;

// Textures
CTexture * TexMenu;
CTexture * TexGame;
CTexture * TexGameOver;

// Square sprites.
CSquareSprite memuSprite( 0, 0, 255, 255);
CSquareSprite gameoverSprite( 0, 0, 255, 255);

// Ordinary sprites. 
CSprite planeSprite(0.0, 0.0, 160, 80, 90, 64);
CSprite SubMarineSprite(100.0, -100.0, 0, 64, 32, 96);

// 10 Sprites to shoot with.
CSprite * spriteShotArr[10];

// Counter to make a delay between shots...
int shotCounter = 0;


// -=*=-=*=-=*=-=*=-=*=-=*=-=*=- Start Program -=*=-=*=-=*=-=*=-=*=-

int main(void)
{

	// Initialise the Memory Manager and allocate 640 4K blocks, this is actually 2.5 MB...
	Manager.Initialise(640);

	//Get Handle sps2
	iSPS2Desc = Manager.GetSPS2Handle();
	
	// Initilize Sprite offset.
	// Only necessary if you actually use the CSquareSprite class...
	CSquareSprite::setSpriteOffset();


	// Initialise the Pad connected to port 1 of the PS2, we initilize it as analogue so we can read the pressure.
	if(pad_init (PAD_0, PAD_INIT_LOCK | PAD_INIT_ANALOGUE | PAD_INIT_PRESSURE ) == 0)
	{
		printf("Unable to initialise the pads\n");
		pad_cleanup (PAD_0);
		exit(0);
	}

	// The shoot sprites...
	for (int i=0; i < 10 ; i++) {
		spriteShotArr[i] = new CSprite(-500.0, -500.0, 206, 4, 12, 24);
		spriteShotArr[i]->setVisible(false);
	}


	// Submarine Animation.
	SubMarineSprite.animationSetTextureDirection(0);
	SubMarineSprite.animationSetFramesInAnimation(5);
	SubMarineSprite.animationSetFramePos(0);
	SubMarineSprite.animationSetFrameRepeat(1);
	SubMarineSprite.animationSetFrameRepeatBackAndForth(1);
	SubMarineSprite.animationSetFrameDelay(30);
	
	// The textures
	TexMenu = new CTexture(iSPS2Desc);
	TexGame = new CTexture(iSPS2Desc);
	TexGameOver = new CTexture(iSPS2Desc);

	// Load menu bitmap
	if(!TexMenu->LoadBitmap("intro1945.bmp", true, false))
	{
		printf("Failed to load texture intro1945.bmp.");
		return -1;
	}
	// Load game bitmap
	if(!TexGame->LoadBitmap("game1945.bmp", 0, 67, 171))
	{
		printf("Failed to load texture game1945.bmp.");
		return -1;
	}
	// Load game over bitmap
	if(!TexGameOver->LoadBitmap("over1945.bmp", true, false))
	{
		printf("Failed to load texture over1945.bmp.");
		return -1;
	}

	// Initialize game mode and menu
	gameMode = GAME_MODE_MENU;
	MenuInit();
	
// -=*=-=*=-=*=-=*=-=*=-=*=-=*=- Main Loop -=*=-=*=-=*=-=*=-=*=-

	do
	{

		// Read the pad.
		pad_update (PAD_0);

		// This is a general counter used for the menu etc...
		counter++;
		// Reset counter after 10 000 cycles, however the menu will reset this after 1000...
		if (counter > 10000) {
			counter = 0;
		} 

		// Switch the game mode... 
		switch (gameMode) { 
		case GAME_MODE_MENU: 
			sps2UScreenClear(0x00, 0x00, 0x00);
			GameMenu(); 
		break; 
		case GAME_MODE_PLAYING: 
			sps2UScreenClear(0, 67, 171);
			GamePlaying(); 
		break; 
		case GAME_MODE_GAMEOVER: 
			sps2UScreenClear(0x00, 0x00, 0x00);
			GameOver();
		break;
		default:
			sps2UScreenClear(0x00, 0x00, 0x00);
			GameMenu();
		break;
		}

		// Display the buffer we were just drawing to
		sps2UScreenSwap(); 

	// Press start and select together to end the loop, this can be done anytime in the game...
	}while(!((pad[0].buttons & PAD_START)&&(pad[0].buttons & PAD_SELECT)));

// -=*=-=*=-=*=-=*=-=*=-=*=-=*=- Cleanup -=*=-=*=-=*=-=*=-=*=-
	
	// Shutdown the screen
	sps2UScreenShutdown();

	// Release SPS2
	sps2Release(iSPS2Desc);

	// Release pad.
	pad_cleanup (PAD_0);

	// Clean up sprites.
	for (int i=0; i < 10 ; i++) {
		delete spriteShotArr[i];
	}

	// Clean up textures
	delete TexMenu;
	delete TexGame;
	delete TexGameOver;

	return 0;
}

// -=*=-=*=-=*=-=*=-=*=-=*=-=*=- End of program -=*=-=*=-=*=-=*=-=*=- 


// Initialize the menu 
void MenuInit(void){ 
	counter = 0; 
	//upload the menu texture, false because we are not going to rotate the textures.
	TexMenu->Upload(sps2UScreenGetFirstFreeGSPage(), false);
	// Render the sprite in the middle of the screen. 
	memuSprite.setPos(192,112); 
}
 
 
// Called once per frame for the menu 
void GameMenu(void) { 
 
	// Print static text... 
	sps2UPrintf(0,"\377808080\376(190,20)PRESS <Start>.");
	sps2UPrintfRender();
 
	// Render the sprite. 
	// Get start of memory. 
	pPrimitiveMem = (char *)Manager.GetPrim1Base(); 
	// Add Sprite to memory. Giftag and primitive data. 
	memuSprite.addSpriteToMem(pPrimitiveMem, 100); 
	// Send off to GS. 
	Manager.FirePacketToGS((char *)Manager.GetPrim1Base(), pPrimitiveMem, (char *)Manager.GetPrim1DmaBase()); 
 
	// Check to see if we started playing... 
	if(pad[0].pressed & PAD_START) { 
		GameInit(); 
		gameMode = 	GAME_MODE_PLAYING; 
	} 
	return; 
} 
 
// Init the game 
void GameInit(void) { 
	// Zero score 
	Score = 0; 
	Score1 = 0; 
	Score2 = 0; 
	counter = 0; 
	//upload the game texture, true because we are going to rotate the textures.
	TexGame->Upload(sps2UScreenGetFirstFreeGSPage(), true); 
 
	// Uncomment to make a "Ghost plane"... ;-) 
//	planeSprite.setColor( 0x40, 0x40, 0x40, 0x40);
 
} 
 
void GamePlaying(void) { 
 
	// Print static text... 
	sps2UPrintf(0,"\377808080\376(50,10)SCORE <1>");
	sps2UPrintf(0,"\377808080\376(60,25)%d", Score1); 

	sps2UPrintf(0,"\377808080\376(250,10)HI-SCORE");
	sps2UPrintf(0,"\377808080\376(260,25)%d", hiScore); 

	sps2UPrintf(0,"\377808080\376(450,10)SCORE <2>"); 
	sps2UPrintf(0,"\377808080\376(460,25)%d", Score2); 
 
	sps2UPrintfRender();
 
	// Animate submarine.
	SubMarineSprite.animationUpdate();

	if (shotCounter > 0) {
		shotCounter--;
	}

	// Did we hit the X button 
	if (pad[0].buttons & PAD_CROSS) {
		if (shotCounter == 0) {
			for (int i=0; i < 10 ; i++) {
				if (!spriteShotArr[i]->getVisible()) {
					spriteShotArr[i]->setAngle(planeSprite.getAngle());
					spriteShotArr[i]->setPos( planeSprite.getXPos(), planeSprite.getYPos());
					spriteShotArr[i]->setVisible(true);
					spriteShotArr[i]->moveDirection(6.0);
					shotCounter = 10;
					i =10;
				}
			}
		}
	}


	// Did we hit the Square button 
	if (pad[0].buttons & PAD_SQUARE) { 
		planeSprite.setScaleX(planeSprite.getScaleX() + 0.03);
		planeSprite.setScaleY(planeSprite.getScaleY() + 0.03);
	} 

	// Did we hit the Square button 
	if (pad[0].buttons & PAD_CIRCLE) { 
		planeSprite.setScaleX(planeSprite.getScaleX() - 0.03);
		planeSprite.setScaleY(planeSprite.getScaleY() - 0.03);
	} 



	// Animate shot 
	for (int i=0; i < 10 ; i++) {
		if (spriteShotArr[i]->getVisible()) {
			spriteShotArr[i]->moveDirection(6.0);
			if (spriteShotArr[i]->getXPos() > 350.0 || spriteShotArr[i]->getXPos() < -350.0 || spriteShotArr[i]->getYPos() > 270.0 || spriteShotArr[i]->getYPos() < -270.0) {
				spriteShotArr[i]->setVisible(false);
			}
		}
	}

	// Did we hit ??? 
	for (int i=0; i < 10 ; i++) {
		if (spriteShotArr[i]->inside(SubMarineSprite)) {
			spriteShotArr[i]->setVisible(false);
			Score1++;
		}
	}

	// Steer plane...
	if (pad[0].buttons & PAD_RIGHT) {
		planeSprite.addAngle(0.03);
	}

	if (pad[0].buttons & PAD_LEFT) { 
		planeSprite.addAngle(-0.03); 
	} 

	// Move plane 
	planeSprite.moveDirection(2.5); 

	// Wrap plane around 
	if (planeSprite.getXPos() > 350.0) { 
		planeSprite.setPos( -350.0, planeSprite.getYPos()); 
		 
	} 

	if (planeSprite.getXPos() < -350.0) { 
		planeSprite.setPos( 350.0, planeSprite.getYPos()); 
		 
	} 

	if (planeSprite.getYPos() < -270.0) { 
		planeSprite.setPos( planeSprite.getXPos(), 270.0); 
		 
	} 

	if (planeSprite.getYPos() > 270.0) { 
		planeSprite.setPos( planeSprite.getXPos(), -270.0); 
		 
	} 

	// Render sprite 
	// Memory start. 
	pPrimitiveMem = (char *)Manager.GetPrim1Base();  
	// Add sprites to memory. 
	SubMarineSprite.addSpriteToMem(pPrimitiveMem, 200); 
	planeSprite.addSpriteToMem(pPrimitiveMem, 1000); 

	for (int i=0; i < 10 ; i++) {
		spriteShotArr[i]->addSpriteToMem(pPrimitiveMem, 500);
	}

	// Send to GS. 
	Manager.FirePacketToGS((char *)Manager.GetPrim1Base(), pPrimitiveMem, (char *)Manager.GetPrim1DmaBase()); 


	// Game over if you press the Triangle. 
	if (pad[0].pressed & PAD_TRI)   { 
		GameOverInit(); 
		gameMode = 	GAME_MODE_GAMEOVER; 
	} 
	return; 
} 
 
void GameOverInit(void) { 
	counter = 0; 
	//upload the game over texture, false because we are not going to rotate the textures.
	TexGameOver->Upload(sps2UScreenGetFirstFreeGSPage(), false);
	gameoverSprite.setPos(192,112); 
}

void GameOver(void) { 
 
	// Static text. 
	sps2UPrintf(0,"\377808080\376(190,20)PRESS SQUARE TO CONTINUE."); 
 
	// Render it. 
	sps2UPrintfRender();
 
	// Render sprite. 
	pPrimitiveMem = (char *)Manager.GetPrim1Base();  
	gameoverSprite.addSpriteToMem(pPrimitiveMem, 100); 
	Manager.FirePacketToGS((char *)Manager.GetPrim1Base(), pPrimitiveMem, (char *)Manager.GetPrim1DmaBase()); 
 
	// Test to see if a certain time has elepsed or the user pressed the Square. 
	if ((pad[0].pressed & PAD_SQUARE)| counter > 1000) { 
		MenuInit(); 
		gameMode = 	GAME_MODE_MENU; 
	} 
	return; 
}
 
 
